<?php

namespace App\Http\Controllers;

use App\Models\Certificado;
use App\Models\Miembro;
use App\Models\Iglesia;
use Barryvdh\DomPDF\Facade\Pdf;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use setasign\Fpdi\TcpdfFpdi;

class CertificadoController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        // Obtener los parámetros de búsqueda y filtro
        $searchNumero = $request->get('numero');
        $searchFolio = $request->get('folio');
        $tipo = $request->get('tipo');
        $searchFecha = $request->get('fecha');
    
        // Construir la consulta base
        $query = Certificado::query();
    
        // Obtener el usuario autenticado
        $user = Auth::user();
    
        // Verificar si el usuario es secretario
        if ($user->hasRole('Secretario')) {
            $secretario = $user->secretario;
            
            // Filtrar solo los certificados de la misma iglesia que el secretario
            $query->where('iglesia_id', $secretario->iglesia_id);
        }
    
        // Aplicar filtros adicionales si existen
        if ($searchNumero) {
            $query->where('numero', 'like', '%' . $searchNumero . '%');
        }
    
        if ($searchFolio) {
            $query->where('folio', 'like', '%' . $searchFolio . '%');
        }
    
        if ($tipo) {
            $query->where('tipo', $tipo);
        }
    
        if ($searchFecha) {
            $query->where(function($q) use ($searchFecha) {
                $q->where('fecha', $searchFecha)
                  ->orWhere('fecha_matrimonio', $searchFecha);
            });
        }
    
        // Ordenar por el último agregado primero
        $certificados = $query->orderBy('created_at', 'desc')->get();
    
        return view('certificados.index', compact('certificados', 'searchNumero', 'searchFolio', 'tipo', 'searchFecha'));
    }
    

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        // Obtener el usuario autenticado
        $user = Auth::user();
    
        // Verificar si el usuario es secretario
        if ($user->hasRole('Secretario')) {
            $secretario = $user->secretario;
    
            // Obtener los miembros de la misma iglesia que el secretario
            $miembros = Miembro::where('iglesia_id', $secretario->iglesia_id)->get();
            $iglesias = Iglesia::where('id', $secretario->iglesia_id)->get();
        } else if ($user->hasRole('Administrador')) {
            // Si el usuario es administrador, obtener todos los miembros e iglesias
            $miembros = Miembro::all();
            $iglesias = Iglesia::all();
        } else {
            // Redirigir si no tiene permisos para acceder
            return redirect()->route('home')->with('error', 'No tienes permiso para registrar certificados.');
        }
    
        return view('certificados.create', compact('miembros', 'iglesias'));
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // Validaciones comunes para todos los certificados
        $request->validate([
            'tipo' => 'required|in:bautismo,matrimonio,consagracion',
            'fecha' => 'nullable|date',
        ]);
    
        // Inicialización de datos comunes con `numero` vacío y `folio` asignado según tipo de certificado
        $data = [
            'tipo' => $request->tipo,
            'numero' => null,
            'folio' => $request->tipo === 'bautismo' ? '2' : ($request->tipo === 'matrimonio' ? '1' : '3'),
            'iglesia_id' => $request->iglesia_id,
            'fecha' => $request->fecha,
        ];
    
        // Validaciones y asignación específica de campos según el tipo de certificado
        switch ($request->tipo) {
            case 'bautismo':
                $request->validate([
                    'miembro_id' => 'required|exists:miembros,id',
                    'iglesia_id' => 'required|exists:iglesias,id',
                    'otorgado' => 'nullable|string|max:255',
                    'lugar' => 'nullable|string|max:255',
                ]);
                $data['miembro_id'] = $request->miembro_id;
                $data['otorgado'] = $request->otorgado;
                $data['lugar'] = $request->lugar;
                break;
    
            case 'matrimonio':
                $request->validate([
                    'miembro_senor_id' => 'nullable|exists:miembros,id',
                    'senor' => 'nullable|string|max:255',
                    'miembro_senora_id' => 'nullable|exists:miembros,id',
                    'senora' => 'nullable|string|max:255',
                    'iglesia_id' => 'required|exists:iglesias,id',
                    'notaria_numero' => 'nullable|string|max:255',
                    'libro_numero' => 'nullable|string|max:255',
                    'fecha_matrimonio' => 'required|date',
                ]);
                if (!$request->miembro_senor_id && !$request->senor) {
                    return back()->withErrors(['miembro_senor_id' => 'Debe seleccionar un miembro o ingresar manualmente el nombre del Señor.']);
                }
                if (!$request->miembro_senora_id && !$request->senora) {
                    return back()->withErrors(['miembro_senora_id' => 'Debe seleccionar un miembro o ingresar manualmente el nombre de la Señora.']);
                }
                $data['miembro_senor_id'] = $request->miembro_senor_id;
                $data['senor'] = $request->miembro_senor_id ? null : $request->senor;
                $data['miembro_senora_id'] = $request->miembro_senora_id;
                $data['senora'] = $request->miembro_senora_id ? null : $request->senora;
                $data['notaria_numero'] = $request->notaria_numero;
                $data['libro_numero'] = $request->libro_numero;
                $data['fecha_matrimonio'] = $request->fecha_matrimonio;
                break;
    
            case 'consagracion':
                $request->validate([
                    'miembro_id' => 'required|exists:miembros,id',
                    'iglesia_id' => 'required|exists:iglesias,id',
                    'padre' => 'required|string|max:255',
                    'madre' => 'required|string|max:255',
                    'fecha' => 'nullable|date',
                ]);
                $data['miembro_id'] = $request->miembro_id;
                $data['padre'] = $request->padre;
                $data['madre'] = $request->madre;
                break;
        }
    
        Certificado::create($data);
        return redirect()->route('certificados.index')->with('success', 'Certificado creado exitosamente.');
    }
    
    

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Certificado  $certificado
     * @return \Illuminate\Http\Response
     */
    public function show(Certificado $certificado)
    {
        return view('certificados.show', compact('certificado'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Certificado  $certificado
     * @return \Illuminate\Http\Response
     */
    public function edit(Certificado $certificado)
    {
        // Obtener el usuario autenticado
        $user = Auth::user();
    
        // Verificar si el usuario es secretario
        if ($user->hasRole('Secretario')) {
            $secretario = $user->secretario;
    
            // Obtener solo los miembros de la misma iglesia que el secretario
            $miembros = Miembro::where('iglesia_id', $secretario->iglesia_id)->get();
            $iglesias = Iglesia::where('id', $secretario->iglesia_id)->get();
        } else if ($user->hasRole('Administrador')) {
            // Si el usuario es administrador, obtener todos los miembros e iglesias
            $miembros = Miembro::all();
            $iglesias = Iglesia::all();
        } else {
            // Redirigir si no tiene permisos para acceder
            return redirect()->route('home')->with('error', 'No tienes permiso para editar certificados.');
        }
    
        // Seleccionar el formulario correcto según el tipo de certificado
        switch ($certificado->tipo) {
            case 'bautismo':
                $view = 'certificados.edit_bautismo';
                break;
            case 'matrimonio':
                $view = 'certificados.edit_matrimonio';
                break;
            case 'consagracion':
                $view = 'certificados.edit_consagracion';
                break;
            default:
                return redirect()->route('certificados.index')->with('error', 'Tipo de certificado no válido.');
        }
    
        return view($view, compact('certificado', 'miembros', 'iglesias'));
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Certificado  $certificado
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Certificado $certificado)
    {
        // Validaciones comunes y específicas
        $request->validate([
            'numero' => 'nullable|string|max:255',
            'folio' => 'nullable|string|max:255',
            'fecha' => 'nullable|date',
        ]);
    
        switch ($certificado->tipo) {
            case 'bautismo':
                $request->validate([
                    'miembro_id' => 'required|exists:miembros,id',
                    'otorgado' => 'nullable|string|max:255',
                    'lugar' => 'nullable|string|max:255',
                ]);
                $certificado->update($request->all());
                break;
    
            case 'matrimonio':
                $request->validate([
                    'senor' => 'nullable|string|max:255',
                    'senora' => 'nullable|string|max:255',
                    'notaria_numero' => 'nullable|string|max:255',
                    'libro_numero' => 'nullable|string|max:255',
                    'fecha_matrimonio' => 'required|date',
                ]);
                $certificado->update($request->all());
                break;
    
            case 'consagracion':
                $request->validate([
                    'miembro_id' => 'required|exists:miembros,id',
                    'padre' => 'required|string|max:255',
                    'madre' => 'required|string|max:255',
                ]);
                $certificado->update($request->all());
                break;
        }
    
        return redirect()->route('certificados.index')->with('success', 'Certificado actualizado exitosamente.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Certificado  $certificado
     * @return \Illuminate\Http\Response
     */
    public function destroy(Certificado $certificado)
    {
        $certificado->delete();

        return redirect()->route('certificados.index')->with('success', 'Certificado eliminado exitosamente.');
    }




    public function generatePdf($id)
    {
        $certificado = Certificado::findOrFail($id);
        $miembro = $certificado->miembro;
        $iglesia = $certificado->iglesia;
    
        // Seleccionar la vista según el tipo de certificado
        $view = '';
        switch ($certificado->tipo) {
            case 'bautismo':
                $view = 'certificados.pdf_bautismo';
                break;
            case 'matrimonio':
                $view = 'certificados.pdf_matrimonio';
                break;
            case 'consagracion':
                $view = 'certificados.pdf_consagracion';
                break;
        }
    
        // Generar el PDF utilizando la vista seleccionada
        $pdf = PDF::loadView($view, compact('certificado', 'miembro', 'iglesia'))->setPaper('a4', 'landscape');
    
        // Guardar el PDF en una ubicación temporal
        $tempPdfPath = storage_path('app/temp/' . 'certificado_' . $certificado->tipo . '_' . $certificado->id . '.pdf');
        $pdf->save($tempPdfPath);
    
        // Crear una nueva instancia de TCPDF usando FPDI para agregar la firma digital
        $pdfFpdi = new TcpdfFpdi('L', PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
        $pdfFpdi->AddPage();
    
        // Cargar el PDF generado en FPDI
        $pageCount = $pdfFpdi->setSourceFile($tempPdfPath);
        $templateId = $pdfFpdi->importPage(1);
        $pdfFpdi->useTemplate($templateId, 0, 0, 297, 210, true);
    
        // Ruta del archivo de la firma digital
        $p12FilePath = base_path('firma/softoken.p12');
        $p12Password = '6983531Lp.RY'; 
    
        // Cargar el certificado digital
        $pkcs12 = file_get_contents($p12FilePath);
        openssl_pkcs12_read($pkcs12, $certs, $p12Password);
    
        // Obtener la clave privada y el certificado
        $privateKey = $certs['pkey'];
        $cert = $certs['cert'];
    
        // Agregar la firma digital al PDF
        $pdfFpdi->setSignature(
            $cert, 
            $privateKey, 
            $cert, 
            '', 
            2, 
            ['Name' => 'Firma Digital', 'Location' => 'Iglesia', 'Reason' => 'Certificado']
        );
    
        // Guardar el PDF firmado en una ubicación temporal
        $signedPdfPath = storage_path('app/temp/' . 'certificado_firmado_' . $certificado->tipo . '_' . $certificado->id . '.pdf');
        $pdfFpdi->Output($signedPdfPath, 'F');
    
        // Descargar el PDF firmado
        return response()->download($signedPdfPath)->deleteFileAfterSend(true);
    }

}
