<?php

namespace App\Http\Controllers;

use App\Models\Pago;
use App\Models\Certificado;
use App\Models\QR;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use PDF;

class PagoController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        // Base de la consulta
        $query = Certificado::whereIn('tipo', ['matrimonio', 'consagracion'])
            ->with(['pagos' => function ($q) {
                $q->orderBy('fecha_de_pago', 'desc'); // Ordenar pagos más recientes primero
            }, 'pagos.usuario'])
            ->orderByDesc(
                Pago::select('fecha_de_pago')
                    ->whereColumn('certificados.id', 'pagos.certificado_id')
                    ->latest()
                    ->take(1)
            ); // Ordenar certificados por el último pago registrado
    
        // Aplicar filtros
        if ($request->filled('numero')) {
            $query->where('numero', 'like', '%' . $request->numero . '%');
        }
        if ($request->filled('folio')) {
            $query->where('folio', 'like', '%' . $request->folio . '%');
        }
        if ($request->filled('fecha_inicio') && $request->filled('fecha_fin')) {
            $fechaInicio = \Carbon\Carbon::parse($request->fecha_inicio)->startOfDay();
            $fechaFin = \Carbon\Carbon::parse($request->fecha_fin)->endOfDay();
    
            $query->whereHas('pagos', function ($q) use ($fechaInicio, $fechaFin) {
                $q->whereBetween('fecha_de_pago', [$fechaInicio, $fechaFin]);
            });
        }
        if ($request->filled('estado_pago')) {
            if ($request->estado_pago === 'completo') {
                $query->whereHas('pagos', function ($q) {
                    $q->where('estado', 'completo');
                });
            } elseif ($request->estado_pago === 'incompleto') {
                $query->whereDoesntHave('pagos', function ($q) {
                    $q->where('estado', 'completo');
                });
            }
        }
    
        // Obtener certificados y pagos
        $certificados = $query->get();
    
        // Dividir en completos e incompletos
        $pagosCompletos = $certificados->filter(function ($certificado) {
            return $certificado->pagos->last() && $certificado->pagos->last()->estado === 'completo';
        });
    
        $pagosIncompletos = $certificados->filter(function ($certificado) {
            return !$certificado->pagos->last() || $certificado->pagos->last()->estado !== 'completo';
        });
    
        return view('pagos.index', compact('pagosCompletos', 'pagosIncompletos'));
    }
    

    
    /**
     * Show the form for creating a new resource.
     *
     * @param int $certificado_id
     * @return \Illuminate\Http\Response
     */
    public function create($certificado_id)
    {
        $certificado = Certificado::findOrFail($certificado_id);
    
        // Determinar el monto a pagar según el tipo de certificado
        $montoAPagar = ($certificado->tipo === 'matrimonio') ? 100 : 30;
    
        // Obtener los pagos relacionados con el certificado
        $pagos = Pago::where('certificado_id', $certificado_id)->get();
    
        // Obtener el último QR con estado 1
        $qr = QR::where('estado', 1)->latest()->first();
    
        return view('pagos.create', compact('certificado', 'montoAPagar', 'pagos', 'qr'));
    }
    
    

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
{
    $request->validate([
        'certificado_id' => 'required|exists:certificados,id',
        'fecha_de_pago' => 'required|date_format:Y-m-d\TH:i',
        'tipo' => 'required|string', // Validación para el campo tipo
    ]);

    $certificado = Certificado::findOrFail($request->certificado_id);

    $montoAPagar = ($certificado->tipo === 'matrimonio') ? 100 : 30;

    $pagosPrevios = $certificado->pagos()->sum('monto_pagado');
    $totalPagado = $pagosPrevios + $montoAPagar;

    $estado = ($totalPagado >= $montoAPagar) ? 'completo' : 'incompleto';

    Pago::create([
        'certificado_id' => $certificado->id,
        'monto_a_pagar' => $montoAPagar,
        'monto_pagado' => $montoAPagar,
        'fecha_de_pago' => $request->fecha_de_pago,
        'usuario_id' => Auth::id(),
        'estado' => $estado,
        'tipo' => $request->tipo, // Asignación del nuevo campo
    ]);

    return redirect()->route('certificados.index')->with('success', 'Pago registrado exitosamente.');
}

    
    

    public function exportPdf(Request $request)
    {
        // Copiar la lógica de los filtros del método `index`
        $query = Certificado::whereIn('tipo', ['matrimonio', 'consagracion'])->with('pagos');
    
        // Filtros por número, folio, estado de pago y fecha
        if ($request->filled('numero')) {
            $query->where('numero', 'like', '%' . $request->numero . '%');
        }
    
        if ($request->filled('folio')) {
            $query->where('folio', 'like', '%' . $request->folio . '%');
        }
    
        if ($request->filled('estado_pago')) {
            if ($request->estado_pago === 'completo') {
                // Filtrar por pagos completos
                $query->whereHas('pagos', function ($q) {
                    $q->where('estado', 'completo');
                });
            } elseif ($request->estado_pago === 'incompleto') {
                // Filtrar certificados sin pagos completos
                $query->whereDoesntHave('pagos', function ($q) {
                    $q->where('estado', 'completo');
                });
            }
        }
    
        if ($request->filled('fecha_inicio') && $request->filled('fecha_fin')) {
            $fechaInicio = \Carbon\Carbon::parse($request->fecha_inicio)->startOfDay();
            $fechaFin = \Carbon\Carbon::parse($request->fecha_fin)->endOfDay();
    
            $query->whereHas('pagos', function ($q) use ($fechaInicio, $fechaFin) {
                $q->whereBetween('fecha_de_pago', [$fechaInicio, $fechaFin]);
            });
        }
    
        // Obtener los certificados filtrados
        $certificados = $query->get();
    
        // Filtrar certificados con pagos completos
        $pagosCompletos = $certificados->filter(function ($certificado) {
            $ultimoPago = $certificado->pagos->last();
            return $ultimoPago && $ultimoPago->estado === 'completo';
        });
    
        // Calcular el monto total
        $montoTotal = $pagosCompletos->sum(function ($certificado) {
            return $certificado->pagos->sum('monto_pagado');
        });
    
        // Cargar la vista de PDF con los pagos completos
        $pdf = PDF::loadView('pagos.pdf', compact('pagosCompletos', 'montoTotal'));
    
        // Descargar el PDF con el reporte
        return $pdf->download('reporte_pagos_completos_' . now()->format('Ymd_His') . '.pdf');
    }
    
    
    
}
