<?php


namespace App\Http\Controllers;

use App\Models\Certificado;
use App\Models\Pago;
use Illuminate\Http\Request;
use Barryvdh\DomPDF\Facade\Pdf;
use Carbon\Carbon;

class ReporteController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Mostrar el compendio de reportes disponibles.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        // Total de certificados emitidos
        $totalCertificados = Certificado::count();

        // Certificados emitidos por iglesia y regional
        $certificadosTotalesData = Certificado::select('iglesias.nombre as iglesia', 'regionales.nombre as regional', \DB::raw('count(certificados.id) as total'))
            ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
            ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
            ->groupBy('iglesias.nombre', 'regionales.nombre')
            ->get();

        // Certificados emitidos por tipo (para la lista en la tarjeta)
        $certificadosPorTipo = Certificado::select('tipo', \DB::raw('count(*) as total'))
            ->groupBy('tipo')
            ->get();

        // Certificados emitidos por tipo, iglesia y regional (para el modal de certificados por tipo)
        $certificadosPorTipoData = Certificado::select('tipo', 'iglesias.nombre as iglesia', 'regionales.nombre as regional', \DB::raw('count(certificados.id) as total'))
            ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
            ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
            ->groupBy('tipo', 'iglesias.nombre', 'regionales.nombre')
            ->get();

        // Pagos realizados por iglesia y regional (para el modal de pagos por iglesia)
        $pagosPorIglesia = Pago::select('certificado_id', \DB::raw('sum(monto_pagado) as total'))
            ->with(['certificado', 'certificado.iglesia'])
            ->groupBy('certificado_id')
            ->get();

        // Pagos por iglesia y regional para modal con resumen
        $pagosPorIglesiaData = Pago::select('iglesias.nombre as iglesia', 'regionales.nombre as regional', \DB::raw('sum(pagos.monto_pagado) as total_pagado'))
            ->join('certificados', 'pagos.certificado_id', '=', 'certificados.id')
            ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
            ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
            ->groupBy('iglesias.nombre', 'regionales.nombre')
            ->get();

        return view('reportes.index', compact(
            'totalCertificados',
            'certificadosTotalesData',
            'certificadosPorTipo',
            'certificadosPorTipoData',
            'pagosPorIglesia',
            'pagosPorIglesiaData'
        ));
    }

    /**
     * Reporte de total de certificados emitidos.
     *
     * @return \Illuminate\Http\Response
     */
    public function totalCertificados(Request $request)
    {
        // Filtrar por fecha usando created_at si se proporciona
        $fechaInicio = $request->input('fecha_inicio', '2000-01-01');
        $fechaFin = $request->input('fecha_fin', now());
    
        // Obtener los certificados agrupados por iglesia y regional con created_at
        $certificadosTotalesData = Certificado::select('iglesias.nombre as iglesia', 'regionales.nombre as regional', \DB::raw('count(certificados.id) as total'), 'certificados.created_at')
            ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
            ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
            ->whereBetween('certificados.created_at', [$fechaInicio, $fechaFin])
            ->groupBy('iglesias.nombre', 'regionales.nombre', 'certificados.created_at')
            ->get();
    
        return view('reportes.total_certificados', compact('certificadosTotalesData'));
    }
    

    /**
     * Reporte de certificados emitidos por tipo, iglesia y regional.
     */
    public function certificadosPorTipo(Request $request)
    {
        $fechaInicio = $request->input('fecha_inicio', '2000-01-01');
        $fechaFin = $request->input('fecha_fin', now());
    
        $certificadosPorTipoData = Certificado::select('tipo', 'iglesias.nombre as iglesia', 'regionales.nombre as regional', \DB::raw('count(certificados.id) as total'), 'certificados.created_at')
            ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
            ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
            ->whereBetween('certificados.created_at', [$fechaInicio, $fechaFin])
            ->groupBy('tipo', 'iglesias.nombre', 'regionales.nombre', 'certificados.created_at')
            ->get();
    
        return view('reportes.certificados_por_tipo', compact('certificadosPorTipoData'));
    }
    
    /**
     * Reporte de pagos realizados por iglesia y regional.
     */
    public function pagosPorIglesia(Request $request)
    {
        $fechaInicio = $request->input('fecha_inicio', '2000-01-01');
        $fechaFin = $request->input('fecha_fin', Carbon::now());
    
        $pagosPorIglesiaData = Pago::select('iglesias.nombre as iglesia', 'regionales.nombre as regional', \DB::raw('sum(pagos.monto_pagado) as total_pagado'), 'pagos.created_at')
            ->join('certificados', 'pagos.certificado_id', '=', 'certificados.id')
            ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
            ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
            ->whereBetween('pagos.created_at', [$fechaInicio, $fechaFin])
            ->groupBy('iglesias.nombre', 'regionales.nombre', 'pagos.created_at')
            ->get();
    
        return view('reportes.pagos_por_iglesia', compact('pagosPorIglesiaData'));
    }
    
    public function certificadosDetallado(Request $request)
    {
        $fechaInicio = $request->input('fecha_inicio');
        $fechaFin = $request->input('fecha_fin');
        
        $certificadosBautismo = Certificado::where('tipo', 'bautismo')
            ->when($fechaInicio && $fechaFin, function ($query) use ($fechaInicio, $fechaFin) {
                return $query->whereBetween('fecha', [$fechaInicio, $fechaFin]);
            })
            ->with(['miembro', 'iglesia.regional'])
            ->get();
    
        $certificadosMatrimonio = Certificado::where('tipo', 'matrimonio')
            ->when($fechaInicio && $fechaFin, function ($query) use ($fechaInicio, $fechaFin) {
                return $query->whereBetween('fecha_matrimonio', [$fechaInicio, $fechaFin]);
            })
            ->with(['senorMiembro', 'senoraMiembro', 'iglesia.regional'])
            ->get();
    
        $certificadosConsagracion = Certificado::where('tipo', 'consagracion')
            ->when($fechaInicio && $fechaFin, function ($query) use ($fechaInicio, $fechaFin) {
                return $query->whereBetween('fecha', [$fechaInicio, $fechaFin]);
            })
            ->with(['miembro', 'iglesia.regional'])
            ->get();
    
        return view('reportes.certificados_detallado', compact('certificadosBautismo', 'certificadosMatrimonio', 'certificadosConsagracion'));
    }
    

    
    /**
     * Descargar reporte en PDF.
     *
     * @param string $tipoReporte
     * @return \Illuminate\Http\Response
     */
    public function descargarReporte($tipoReporte, Request $request)
    {
        // Obtén las fechas de inicio y fin del filtro, usa valores predeterminados si no se envían
        $fechaInicio = $request->input('fecha_inicio', '2000-01-01');
        $fechaFin = $request->input('fecha_fin', \Carbon\Carbon::now());
    
        $data = [];
        $view = ''; // Inicializa la variable $view para evitar el error
    
        switch ($tipoReporte) {
            case 'certificados_por_tipo':
                // Consultas para los datos necesarios
                $data['certificadosPorTipo'] = Certificado::select('tipo', \DB::raw('count(*) as total'))
                    ->whereBetween('created_at', [$fechaInicio, $fechaFin])
                    ->groupBy('tipo')
                    ->get();
    
                $data['porTipoEIglesia'] = Certificado::select(
                    'tipo', 
                    'iglesias.nombre as iglesia_nombre', 
                    \DB::raw('count(certificados.id) as total')
                )
                ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
                ->whereBetween('certificados.created_at', [$fechaInicio, $fechaFin])
                ->groupBy('tipo', 'iglesias.nombre')
                ->get();
    
                $data['porRegional'] = Certificado::select(
                    'tipo', 
                    'regionales.nombre as regional_nombre', 
                    \DB::raw('count(certificados.id) as total')
                )
                ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
                ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
                ->whereBetween('certificados.created_at', [$fechaInicio, $fechaFin])
                ->groupBy('tipo', 'regionales.nombre')
                ->get();
    
                $view = 'reportes.pdf_certificados_por_tipo'; // La vista que se utilizará para el PDF
              
                break;
                case 'certificados_detallados':
                    // Consultas para los datos necesarios de certificados detallados
                    $data['certificadosBautismo'] = Certificado::where('tipo', 'bautismo')
                        ->whereBetween('fecha', [$fechaInicio, $fechaFin])
                        ->with(['miembro', 'iglesia.regional'])
                        ->get();
        
                    $data['certificadosMatrimonio'] = Certificado::where('tipo', 'matrimonio')
                        ->whereBetween('fecha_matrimonio', [$fechaInicio, $fechaFin])
                        ->with(['senorMiembro', 'senoraMiembro', 'iglesia.regional'])
                        ->get();
        
                    $data['certificadosConsagracion'] = Certificado::where('tipo', 'consagracion')
                        ->whereBetween('fecha', [$fechaInicio, $fechaFin])
                        ->with(['miembro', 'iglesia.regional'])
                        ->get();
        
                    // Pasar las fechas también a la vista del PDF
                    $data['fechaInicio'] = $fechaInicio;
                    $data['fechaFin'] = $fechaFin;
        
                    $view = 'reportes.pdf_certificados_detallado';
                    break;
            case 'total_certificados':
                $data['totalCertificados'] = Certificado::whereBetween('created_at', [$fechaInicio, $fechaFin])->count();
     
                $data['certificadosPorIglesia'] = Certificado::select('iglesias.nombre as iglesia_nombre', \DB::raw('count(certificados.id) as total'))
                    ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
                    ->whereBetween('certificados.created_at', [$fechaInicio, $fechaFin])
                    ->groupBy('iglesias.nombre')
                    ->get();
    
                $data['certificadosPorRegional'] = Certificado::select('regionales.nombre as regional_nombre', \DB::raw('count(certificados.id) as total'))
                    ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
                    ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
                    ->whereBetween('certificados.created_at', [$fechaInicio, $fechaFin])
                    ->groupBy('regionales.nombre')
                    ->get();
    
                $data['totalPorIglesia'] = $data['certificadosPorIglesia']->sum('total');
                $data['totalPorRegional'] = $data['certificadosPorRegional']->sum('total');
    
                $view = 'reportes.pdf_total_certificados';
                break;
    
            case 'pagos_por_iglesia':
                $data['totalesPorTipo'] = Pago::select('certificados.tipo', \DB::raw('sum(pagos.monto_pagado) as total_pagado'))
                    ->join('certificados', 'pagos.certificado_id', '=', 'certificados.id')
                    ->whereBetween('pagos.created_at', [$fechaInicio, $fechaFin])
                    ->groupBy('certificados.tipo')
                    ->get();
    
                $data['porTipoEIglesia'] = Pago::select('certificados.tipo', 'iglesias.nombre as iglesia_nombre', \DB::raw('sum(pagos.monto_pagado) as total_pagado'))
                    ->join('certificados', 'pagos.certificado_id', '=', 'certificados.id')
                    ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
                    ->whereBetween('pagos.created_at', [$fechaInicio, $fechaFin])
                    ->groupBy('certificados.tipo', 'iglesias.nombre')
                    ->get();
    
                $data['porRegional'] = Pago::select('certificados.tipo', 'regionales.nombre as regional_nombre', \DB::raw('sum(pagos.monto_pagado) as total_pagado'))
                    ->join('certificados', 'pagos.certificado_id', '=', 'certificados.id')
                    ->join('iglesias', 'certificados.iglesia_id', '=', 'iglesias.id')
                    ->join('regionales', 'iglesias.regional_id', '=', 'regionales.id')
                    ->whereBetween('pagos.created_at', [$fechaInicio, $fechaFin])
                    ->groupBy('certificados.tipo', 'regionales.nombre')
                    ->get();
    
                $view = 'reportes.pdf_pagos_por_iglesia';
                break;
    
            default:
                return redirect()->route('reportes.index')->with('error', 'Reporte no válido.');
        }
    
        $pdf = Pdf::loadView($view, compact('data'));
    
        return $pdf->download("Reporte_{$tipoReporte}_" . now()->format('Ymd_His') . '.pdf');
    }
    
    
}
