<?php

namespace App\Http\Controllers;

use App\Models\Secretario;
use App\Models\Iglesia;
use Carbon\Carbon;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Database\QueryException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Spatie\Permission\Models\Role;
    
class SecretarioController extends Controller
{    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $search = $request->get('search');
        $iglesiaId = $request->get('iglesia_id');
    
        $query = Secretario::query()->with('iglesia.regional');
    
        if ($search) {
            $query->where(function ($q) use ($search) {
                $q->where('nombre', 'like', '%' . $search . '%')
                  ->orWhere('apellidos', 'like', '%' . $search . '%')
                  ->orWhere('carnet', 'like', '%' . $search . '%')
                  ->orWhere('correo', 'like', '%' . $search . '%');
            });
        }
    
        if ($iglesiaId) {
            $query->where('iglesia_id', $iglesiaId);
        }
    
        // Paginación de los resultados
        $secretarios = $query->paginate(10); // Puedes cambiar el número 10 por la cantidad de registros que desees por página
    
        $iglesias = Iglesia::with('regional')->where('estado', 1)->get();
    
        return view('secretarios.index', compact('secretarios', 'search', 'iglesias', 'iglesiaId'));
    }
    

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $iglesias = Iglesia::where('estado', 1)->pluck('nombre', 'id');
        return view('secretarios.create', compact('iglesias'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'nombre' => 'required|regex:/^[\pL\s\']+$/u|max:255',
            'apellidos' => 'required|regex:/^[\pL\s\']+$/u|max:255',
            'carnet' => 'required|digits_between:6,15|unique:secretario,carnet',
            'telefono' => 'nullable|digits:8',
            'correo' => 'nullable|email|max:255|unique:secretario,correo',
            'direccion' => 'nullable|regex:/^[\pL\d\s\#\.,]+$/u|max:255',
            'fecha_de_nacimiento' => 'required|date|before:' . Carbon::now()->subYears(23)->format('Y-m-d'),
            'iglesia_id' => 'nullable|exists:iglesias,id',
        ], [
            'nombre.required' => 'El nombre es obligatorio.',
            'nombre.regex' => 'El nombre solo puede contener letras, espacios y apóstrofes.',
            'nombre.max' => 'El nombre no puede tener más de 255 caracteres.',
            
            'apellidos.required' => 'Los apellidos son obligatorios.',
            'apellidos.regex' => 'Los apellidos solo pueden contener letras, espacios y apóstrofes.',
            'apellidos.max' => 'Los apellidos no pueden tener más de 255 caracteres.',
            
            'carnet.required' => 'El número de carnet es obligatorio.',
            'carnet.digits_between' => 'El carnet debe tener entre 6 y 15 dígitos.',
            'carnet.unique' => 'El carnet ya está registrado para otro secretario.',
            
            'telefono.digits' => 'El teléfono debe tener exactamente 8 dígitos.',
            
            'correo.email' => 'El correo debe ser una dirección de correo válida.',
            'correo.max' => 'El correo no puede tener más de 255 caracteres.',
            'correo.unique' => 'El correo ya está registrado para otro secretario.',
            
            'direccion.regex' => 'La dirección solo puede contener letras, números, espacios, #, . y ,',
            'direccion.max' => 'La dirección no puede tener más de 255 caracteres.',
            
            'fecha_de_nacimiento.required' => 'La fecha de nacimiento es obligatoria.',
            'fecha_de_nacimiento.date' => 'La fecha de nacimiento debe ser una fecha válida.',
            'fecha_de_nacimiento.before' => 'El secretario debe tener al menos 23 años.',
            
            'iglesia_id.exists' => 'La iglesia seleccionada no existe.',
        ]);
    
        Secretario::create($request->all());
    
        return redirect()->route('secretarios.index')->with('success', 'Secretario creado exitosamente.');
    }
    

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Secretario  $secretario
     * @return \Illuminate\Http\Response
     */
    public function edit(Secretario $secretario)
    {
        $iglesias = Iglesia::where('estado', 1)->pluck('nombre', 'id');
        return view('secretarios.edit', compact('secretario', 'iglesias'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Secretario  $secretario
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Secretario $secretario)
    {
        $request->validate([
            'nombre' => 'required|regex:/^[\pL\s\']+$/u|max:255',
            'apellidos' => 'required|regex:/^[\pL\s\']+$/u|max:255',
            'carnet' => 'required|digits_between:6,15|unique:secretario,carnet,' . $secretario->id,
            'telefono' => 'nullable|digits:8',
            'correo' => 'nullable|email|max:255|unique:secretario,correo,' . $secretario->id,
            'direccion' => 'nullable|regex:/^[\pL\d\s\#\.,]+$/u|max:255',
            'fecha_de_nacimiento' => 'required|date|before:' . Carbon::now()->subYears(23)->format('Y-m-d'),
            'iglesia_id' => 'nullable|exists:iglesias,id',
        ], [
            'carnet.unique' => 'El carnet ya está registrado para otro secretario.',
            'correo.unique' => 'El correo ya está registrado para otro secretario.',
        ]);

        $secretario->update($request->all());

        return redirect()->route('secretarios.index')->with('success', 'Secretario actualizado exitosamente.');
    }

    /**
     * Remove the specified resource from storage (soft delete).
     *
     * @param  \App\Models\Secretario  $secretario
     * @return \Illuminate\Http\Response
     */
    public function destroy(Secretario $secretario)
    {
        $secretario->estado = !$secretario->estado;
        $secretario->save();

        return redirect()->route('secretarios.index')->with('success', 'Estado del secretario actualizado exitosamente.');
    }
    
    public function generateUserForm($secretarioId)
    {
        $secretario = Secretario::find($secretarioId);
        
        // Verificar si  existe
        if (!$secretario) {
            abort(404, 'secretario no encontrado');
        }
        
        return view('secretarios.generate_user', compact('secretario'));
    }
    public function generateUser(Request $request, $secretarioId)
    {
        $secretario = Secretario::find($secretarioId);
    
        // Validar la contraseña
        $request->validate([
            'password' => 'required|string|min:8|confirmed',
        ]);
    
        try {
            // Crear un nuevo usuario
            $user = new User([
                'name' => $secretario->nombre,
                'email' => $secretario->correo,
                'password' => bcrypt($request->password),
            ]);
    
            // Guardar el nuevo usuario
            $user->save();
    
            // Asociar el usuario con el secretario
            $secretario->user()->associate($user);
            $secretario->save();
    
            // Asignar el rol al usuario
            $roleName = 'Secretario';
            $role = Role::where('name', $roleName)->first();
            if (!$role) {
                $role = Role::create(['name' => $roleName]);
            }
            $user->assignRole($roleName);
    
            $notificacion = 'Usuario generado correctamente para ' . $secretario->nombre;
            return redirect()->route('secretarios.index')->with('notificacion', $notificacion);
        } catch (QueryException $e) {
            if ($e->errorInfo[1] == 1062) { // Error de clave única
                $customErrorMessage = 'El correo electrónico ya está registrado. Por favor, elige otro.';
            } else {
                $customErrorMessage = 'Error al crear el usuario. Detalles: ' . $e->getMessage();
            }
    
            return redirect()->back()->with('error', $customErrorMessage);
        } catch (ModelNotFoundException $e) {
            $customErrorMessage = 'Error: miembro no encontrado.';
            return redirect()->back()->with('error', $customErrorMessage);
        }
    }
}
