<?php

namespace App\Http\Controllers;

use App\Models\Certificado;
use App\Models\Iglesia;
use App\Models\Miembro;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Database\QueryException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Spatie\Permission\Models\Role;
    

class MiembroController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $search = $request->get('search');
        $iglesiaId = $request->get('iglesia_id');
    
        $query = Miembro::query()->with('iglesia.regional');
    
        if ($search) {
            $query->where(function ($q) use ($search) {
                $q->where('nombre', 'like', '%' . $search . '%')
                  ->orWhere('apellidos', 'like', '%' . $search . '%')
                  ->orWhere('carnet', 'like', '%' . $search . '%')
                  ->orWhere('correo', 'like', '%' . $search . '%');
            });
        }
    
        if ($iglesiaId) {
            $query->where('iglesia_id', $iglesiaId);
        }
    
        // Paginación de los resultados
        $miembros = $query->paginate(10); // Cambia 10 por la cantidad deseada de registros por página
    
        $iglesias = Iglesia::with('regional')->where('estado', 1)->get();
    
        return view('miembros.index', compact('miembros', 'search', 'iglesias', 'iglesiaId'));
    }
    
    

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $iglesias = Iglesia::where('estado', 1)->pluck('nombre', 'id');
        return view('miembros.create', compact('iglesias'));
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'nombre' => 'required|regex:/^[\pL\s\']+$/u|max:255',
            'apellidos' => 'required|regex:/^[\pL\s\']+$/u|max:255',
            'carnet' => 'required|digits_between:6,15|unique:miembros,carnet',
            'complemento' => 'nullable|alpha_num|max:10',
            'extension' => 'required|in:LP,CB,SC,OR,PT,TJ,BN,PA,CH',
            'telefono' => 'nullable|digits:8',
            'correo' => 'nullable|email|max:255|unique:miembros,correo',
            'direccion' => 'nullable|regex:/^[\pL\d\s\#\.]+$/u|max:255',
            'fecha_de_nacimiento' => 'required|date',
            'iglesia_id' => 'nullable|exists:iglesias,id',
        ], [
            'nombre.required' => 'El nombre es obligatorio.',
            'nombre.regex' => 'El nombre solo puede contener letras, espacios y apóstrofes.',
            'nombre.max' => 'El nombre no puede tener más de 255 caracteres.',
            
            'apellidos.required' => 'Los apellidos son obligatorios.',
            'apellidos.regex' => 'Los apellidos solo pueden contener letras, espacios y apóstrofes.',
            'apellidos.max' => 'Los apellidos no pueden tener más de 255 caracteres.',
            
            'carnet.required' => 'El número de carnet es obligatorio.',
            'carnet.digits_between' => 'El carnet debe tener entre 6 y 15 dígitos.',
            'carnet.unique' => 'El número de carnet ya está registrado.',
            
            'complemento.alpha_num' => 'El complemento solo puede contener letras y números.',
            'complemento.max' => 'El complemento no puede tener más de 10 caracteres.',
            
            'extension.required' => 'La extensión es obligatoria.',
            'extension.in' => 'La extensión seleccionada no es válida.',
            
            'telefono.digits' => 'El teléfono debe tener exactamente 8 dígitos.',
            
            'correo.email' => 'El correo debe ser una dirección de correo válida.',
            'correo.max' => 'El correo no puede tener más de 255 caracteres.',
            'correo.unique' => 'El correo ya está registrado.',
            
            'direccion.regex' => 'La dirección solo puede contener letras, números, espacios, # y .',
            'direccion.max' => 'La dirección no puede tener más de 255 caracteres.',
            
            'fecha_de_nacimiento.required' => 'La fecha de nacimiento es obligatoria.',
            'fecha_de_nacimiento.date' => 'La fecha de nacimiento debe ser una fecha válida.',
            
            'iglesia_id.exists' => 'La iglesia seleccionada no existe.',
        ]);
    
        Miembro::create($request->all());
    
        return redirect()->route('miembros.index')->with('success', 'Miembro creado exitosamente.');
    }
    
    

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Miembro  $miembro
     * @return \Illuminate\Http\Response
     */
    public function edit(Miembro $miembro)
    {
        $iglesias = Iglesia::where('estado', 1)->pluck('nombre', 'id');
        return view('miembros.edit', compact('miembro', 'iglesias'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Miembro  $miembro
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Miembro $miembro)
    {
        $request->validate([
            'nombre' => 'required|regex:/^[\pL\s\']+$/u|max:255',
            'apellidos' => 'required|regex:/^[\pL\s\']+$/u|max:255',
            'carnet' => 'required|digits_between:6,15|unique:miembros,carnet,' . $miembro->id,
            'complemento' => 'nullable|alpha_num|max:10',
            'extension' => 'required|in:LP,CB,SC,OR,PT,TJ,BN,PA,CH',
            'telefono' => 'nullable|digits:8',
            'correo' => 'nullable|email|max:255|unique:miembros,correo,' . $miembro->id,
            'direccion' => 'nullable|regex:/^[\pL\d\s\#\.]+$/u|max:255',
            'fecha_de_nacimiento' => 'required|date',
            'iglesia_id' => 'nullable|exists:iglesias,id',
        ]);
    
        $miembro->update($request->all());
    
        return redirect()->route('miembros.index')->with('success', 'Miembro actualizado exitosamente.');
    }
    
    
    

    /**
     * Remove the specified resource from storage (soft delete).
     *
     * @param  \App\Models\Miembro  $miembro
     * @return \Illuminate\Http\Response
     */
    public function destroy(Miembro $miembro)
    {
        $miembro->estado = !$miembro->estado;
        $miembro->save();

        return redirect()->route('miembros.index')->with('success', 'Estado del miembro actualizado exitosamente.');
    }


     
    public function generateUserForm($miembroId)
    {
        $miembro = Miembro::find($miembroId);
        
        // Verificar si  existe
        if (!$miembro) {
            abort(404, 'miembro no encontrado');
        }
        
        return view('miembros.generate_user', compact('miembro'));
    }
    public function generateUser(Request $request, $miembroId)
    {
        $miembro = Miembro::find($miembroId);
    
        // Validar la contraseña
        $request->validate([
            'password' => 'required|string|min:8|confirmed',
        ]);
    
        try {
            // Crear un nuevo usuario
            $user = new User([
                'name' => $miembro->nombre,
                'email' => $miembro->correo,
                'password' => bcrypt($request->password),
            ]);
    
            // Guardar el nuevo usuario
            $user->save();
    
            // Asociar el usuario con el miembro
            $miembro->user()->associate($user);
            $miembro->save();
    
            // Asignar el rol al usuario
            $roleName = 'Miembro';
            $role = Role::where('name', $roleName)->first();
            if (!$role) {
                $role = Role::create(['name' => $roleName]);
            }
            $user->assignRole($roleName);
    
            $notificacion = 'Usuario generado correctamente para ' . $miembro->nombre;
            return redirect()->route('miembros.index')->with('notificacion', $notificacion);
        } catch (QueryException $e) {
            if ($e->errorInfo[1] == 1062) { // Error de clave única
                $customErrorMessage = 'El correo electrónico ya está registrado. Por favor, elige otro.';
            } else {
                $customErrorMessage = 'Error al crear el usuario. Detalles: ' . $e->getMessage();
            }
    
            return redirect()->back()->with('error', $customErrorMessage);
        } catch (ModelNotFoundException $e) {
            $customErrorMessage = 'Error: miembro no encontrado.';
            return redirect()->back()->with('error', $customErrorMessage);
        }
    }
    
    public function dashboard()
    {
        // Obtener el miembro asociado al usuario autenticado
        $miembro = auth()->user()->miembro;
    
        if (!$miembro) {
            return redirect()->back()->with('error', 'No estás registrado como miembro.');
        }
    
        // Obtener los certificados donde el miembro es miembro principal, señor o señora
        $certificados = Certificado::where('miembro_id', $miembro->id)
                        ->orWhere('miembro_senor_id', $miembro->id)
                        ->orWhere('miembro_senora_id', $miembro->id)
                        ->with('miembro', 'senorMiembro', 'senoraMiembro')
                        ->get();
    
        return view('miembros.dashboard', compact('miembro', 'certificados'));
    }
    
    
  
}
